import { AxiosResponse } from 'axios';
/**
 * Determines whether an object is a DavenportError.
 */
export declare function isDavenportError(error: any): error is DavenportError;
/**
 * A generic view document for listing and counting all objects in the database.
 */
export declare const GENERIC_LIST_VIEW: {
    "name": string;
    "map": string;
    "reduce": string;
};
/**
 * Configures a Davenport client and database by validating the CouchDB version, creating indexes and design documents, and then returning a client to interact with the database.
 */
export declare function configureDatabase<DocType extends CouchDoc>(databaseUrl: string, configuration: DatabaseConfiguration<DocType>, options?: ClientOptions): Promise<Client<DocType>>;
/**
 * A client for interacting with a CouchDB instance. Use this when you don't want or need to use the `configureClient` function to create a database and set up design docs or indexes.
 */
export declare class Client<T extends CouchDoc> {
    private baseUrl;
    private databaseName;
    private options;
    constructor(baseUrl: string, databaseName: string, options?: ClientOptions);
    private axios;
    private databaseUrl;
    private getOption(name);
    /**
     * Checks that the Axios response is okay. If not, a DavenPort error is thrown.
     */
    private checkErrorAndGetBody(result);
    /**
     * Find matching documents according to the selector.
     */
    find(options: FindOptions<T>): Promise<T[]>;
    /**
     * Lists documents in the database. Warning: this result WILL list design documents, and it will force the `include_docs` option to false. If you need to include docs, use .listWithDocs.
     */
    listWithoutDocs(options?: ListOptions): Promise<ListResponse<{
        rev: string;
    }>>;
    /**
     * Lists documents in the database. Warning: this result WILL list design documents, and it will force the `include_docs` option to true. If you don't need to include docs, use .listWithoutDocs.
     */
    listWithDocs(options?: ListOptions): Promise<ListResponse<T>>;
    /**
     * Counts all documents in the database. Warning: this result WILL include design documents.
     */
    count(): Promise<number>;
    /**
     * Counts all documents by the given selector. Warning: this uses more memory than a regular count, because it needs to pull in the _id field of all selected documents. For large queries, it's better to create a dedicated view and use the .view function.
     */
    countBySelector(selector: DocSelector<T>): Promise<number>;
    countBySelector(selector: Partial<T>): Promise<number>;
    /**
     * Gets a document with the given id and optional revision id.
     */
    get(id: string, rev?: string): Promise<T>;
    /**
     * Creates a document with a random id. By CouchDB convention, this will only return the id and revision id of the new document, not the document itself.
     */
    post(data: T): Promise<PostPutCopyResponse>;
    /**
     * Updates or creates a document with the given id. By CouchDB convention, this will only return the id and revision id of the new document, not the document itself.
     */
    put(id: string, data: T, rev: string): Promise<PostPutCopyResponse>;
    /**
     * Copies the document with the given id and assigns the new id to the copy. By CouchDB convention, this will only return the id and revision id of the new document, not the document itself.
     */
    copy(id: string, newId: string): Promise<PostPutCopyResponse>;
    /**
     * Deletes the document with the given id and revision id.
     */
    delete(id: string, rev: string): Promise<void>;
    /**
     * Checks that a document with the given id exists.
     */
    exists(id: string): Promise<boolean>;
    /**
     * Checks that a document that matches the field value exists.
     */
    existsByFieldValue(value: any, field: keyof T): Promise<boolean>;
    /**
     * Checks that a document matching the selector exists.
     */
    existsBySelector(selector: DocSelector<T>): Promise<boolean>;
    /**
     * Executes a view with the given designDocName and viewName. Will not reduce by default, pass in the { reduce: true } option to reduce.
     */
    view<DocType>(designDocName: string, viewName: string, options?: ViewOptions): Promise<ViewResult<DocType>>;
    /**
     * Creates the database associated with this client.
     */
    createDb(url?: string): Promise<BasicCouchResponse>;
    /**
     * Deletes the database associated with this client.
     */
    deleteDb(url?: string): Promise<BasicCouchResponse>;
    /**
     * Returns database info for the given database.
     */
    getDbInfo(url?: string): Promise<DbInfo>;
    private encodeOptions(options);
}
export default Client;
export declare class DavenportError extends Error {
    fullResponse: AxiosResponse;
    constructor(message: any, fullResponse: AxiosResponse);
    readonly isDavenport: boolean;
    status: number;
    statusText: string;
    url: string;
    body: any;
}
export interface CouchDoc {
    /**
     * The object's database id.
     */
    _id?: string;
    /**
     * The object's database revision.
     */
    _rev?: string;
}
export interface PostPutCopyResponse {
    id: string;
    rev: string;
}
export interface ViewOptions extends ListOptions {
    reduce?: boolean;
    group?: boolean;
    group_level?: number;
}
export interface ViewResult<DocType> {
    offset?: number;
    total_rows?: number;
    rows: ViewRow<DocType>[];
}
export interface ViewRow<DocType> {
    id?: string;
    key?: any;
    value: DocType;
}
/**
 * Options for listing database results.
 */
export interface ListOptions {
    limit?: number;
    key?: string;
    keys?: string[];
    start_key?: string | number;
    end_key?: string | number;
    inclusive_end?: boolean;
    descending?: boolean;
    skip?: number;
}
export interface AllDocsListResult<T> {
    rows: {
        id: string;
        key: string;
        value: {
            rev: string;
        };
        doc: T;
    }[];
    offset: number;
    total_rows: number;
}
export interface DbSizes {
    file: number;
    external: number;
    active: number;
}
export interface DbOther {
    data_size?: number;
}
export interface DbInfo {
    db_name: string;
    update_seq: string;
    sizes: DbSizes;
    purge_seq: number;
    other?: DbOther;
    doc_del_count: number;
    doc_count: number;
    disk_size: number;
    disk_format_version: number;
    data_size: number;
    compact_running: boolean;
    instance_start_time: number;
}
export interface BasicCouchResponse {
    ok: boolean;
}
export interface ListResponse<T> {
    offset: number;
    total_rows: number;
    rows: T[];
}
export interface FindOptions<T> {
    fields?: string[];
    sort?: Object[];
    limit?: number;
    skip?: number;
    use_index?: Object;
    selector: Partial<T> | DocSelector<T>;
}
export interface CouchDBView {
    map: string;
    reduce?: string;
}
export interface DesignDoc extends CouchDoc {
    views: {
        [name: string]: CouchDBView;
    };
    language: "javascript";
}
export interface DesignDocConfiguration {
    name: string;
    views: ({
        name: string;
    } & CouchDBView)[];
}
export interface DatabaseConfiguration<T> {
    name: string;
    indexes?: (keyof T)[];
    designDocs?: DesignDocConfiguration[];
}
export interface ClientOptions {
    /**
     * Whether the Davenport client should log warnings.
     */
    warnings?: boolean;
    /**
     * Username used to make requests with basic auth.
     */
    username?: string;
    /**
     * Password used to make requests with basic auth.
     */
    password?: string;
}
export interface PropSelector {
    /**
     * Property is equal to this value.
     */
    $eq?: any;
    /**
     * Property is not equal to this value.
     */
    $ne?: any;
    /**
     * Property is greater than this value.
     */
    $gt?: any;
    /**
     * Property is greater than or equal to this value.
     */
    $gte?: any;
    /**
     * Property is less than this value.
     */
    $lt?: any;
    /**
     * Property is lesser than or equal to this value.
     */
    $lte?: any;
}
export declare type DocSelector<T> = Partial<Record<keyof T, PropSelector>>;
